use std::{
    io::Write,
    process::{Command, Stdio},
};

#[derive(thiserror::Error, Debug)]
pub enum QrGenerationError {
    #[error("{0}")]
    Io(#[from] std::io::Error),

    #[error("{0}")]
    StringParse(#[from] std::string::FromUtf8Error)
}

/// Generate a terminal-printable QR code for a given string. Uses the `qrencode` CLI utility.
pub fn qrencode(text: &str) -> Result<String, QrGenerationError> {
    let mut qrencode = Command::new("qrencode")
        .arg("-t")
        .arg("ansiutf8")
        .arg("-m")
        .arg("2")
        .stdin(Stdio::piped())
        .stdout(Stdio::piped())
        .spawn()?;
    if let Some(stdin) = qrencode.stdin.as_mut() {
        stdin.write_all(text.as_bytes())?;
    }
    let output = qrencode.wait_with_output()?;
    let result = String::from_utf8(output.stdout)?;
    Ok(result)
}
